package api

import (
	"chromagies/src/database"
	"chromagies/src/profiler"
	"fmt"
	"net/http"
	"time"

	"github.com/gorilla/mux"
)

const (
	profiler_API_PROCESS_REQUEST string = "API Process Request"
)

var apiRootPath string = "/api/v1"

func Init(router *mux.Router) {

	profiler.Register(profiler_API_PROCESS_REQUEST, 10000, "µs")

	router.HandleFunc(apiRootPath, apiRoot).Methods("GET")

	routerAuth := router.PathPrefix(apiRootPath + "/auth").Subrouter()
	routerPage := router.PathPrefix(apiRootPath + "/page").Subrouter()
	routerUser := router.PathPrefix(apiRootPath + "/user").Subrouter()
	routerTag := router.PathPrefix(apiRootPath + "/tag").Subrouter()
	routerDebug := router.PathPrefix(apiRootPath + "/debug").Subrouter()

	routerAuth.HandleFunc("", apiAuth)
	routerAuth.HandleFunc("/login", apiAuthLogin)
	routerAuth.HandleFunc("/logout", apiAuthLogout)

	routerPage.HandleFunc("", apiPage)
	routerPage.HandleFunc("/{folder}", apiPageFolder)
	routerPage.HandleFunc("/{folder}/{page}", apiPageFolderPage)
	routerPage.HandleFunc("/{folder}/{page}/content", apiPageFolderPageContent)

	routerUser.HandleFunc("", apiUser)
	routerUser.HandleFunc("/{name}", apiUserName)

	routerTag.HandleFunc("", apiTag)
	routerTag.HandleFunc("/{name}", apiTagName)

	routerDebug.HandleFunc("/profiler", apiDebugProfiler)
	routerDebug.HandleFunc("/profiler/{name}", apiDebugProfilerName)
}

// API ROOT

func apiRoot(w http.ResponseWriter, r *http.Request) {
	tStart := time.Now()
	fmt.Fprintf(w, "API ROOT")
	profiler.Add(profiler_API_PROCESS_REQUEST, time.Duration(time.Since(tStart).Microseconds()))
}

// API Auth

func apiAuth(w http.ResponseWriter, r *http.Request) {
	tStart := time.Now()
	fmt.Fprintf(w, "API Auth")
	profiler.Add(profiler_API_PROCESS_REQUEST, time.Duration(time.Since(tStart).Microseconds()))
}

func apiAuthLogin(w http.ResponseWriter, r *http.Request) {
	tStart := time.Now()
	fmt.Fprintf(w, "API Auth Login")
	database.Ping()
	profiler.Add(profiler_API_PROCESS_REQUEST, time.Duration(time.Since(tStart).Microseconds()))
}

func apiAuthLogout(w http.ResponseWriter, r *http.Request) {
	tStart := time.Now()
	fmt.Fprintf(w, "API Auth Logout")
	profiler.Add(profiler_API_PROCESS_REQUEST, time.Duration(time.Since(tStart).Microseconds()))
}

// API Page

func apiPage(w http.ResponseWriter, r *http.Request) {
	tStart := time.Now()
	fmt.Fprintf(w, "API Page")
	profiler.Add(profiler_API_PROCESS_REQUEST, time.Duration(time.Since(tStart).Microseconds()))
}

func apiPageFolder(w http.ResponseWriter, r *http.Request) {
	tStart := time.Now()
	vars := mux.Vars(r)
	fmt.Fprintf(w, "API Page Folder(%s)", vars["folder"])
	profiler.Add(profiler_API_PROCESS_REQUEST, time.Duration(time.Since(tStart).Microseconds()))
}

func apiPageFolderPage(w http.ResponseWriter, r *http.Request) {
	tStart := time.Now()
	vars := mux.Vars(r)
	fmt.Fprintf(w, "API Page Folder(%s) Page(%s)", vars["folder"], vars["page"])
	profiler.Add(profiler_API_PROCESS_REQUEST, time.Duration(time.Since(tStart).Microseconds()))
}

func apiPageFolderPageContent(w http.ResponseWriter, r *http.Request) {
	tStart := time.Now()
	vars := mux.Vars(r)
	fmt.Fprintf(w, "API Page Folder(%s) Page(%s) Content", vars["folder"], vars["page"])
	profiler.Add(profiler_API_PROCESS_REQUEST, time.Duration(time.Since(tStart).Microseconds()))
}

// API User

func apiUser(w http.ResponseWriter, r *http.Request) {
	tStart := time.Now()
	fmt.Fprintf(w, "API User")
	profiler.Add(profiler_API_PROCESS_REQUEST, time.Duration(time.Since(tStart).Microseconds()))
}

func apiUserName(w http.ResponseWriter, r *http.Request) {
	tStart := time.Now()
	vars := mux.Vars(r)
	fmt.Fprintf(w, "API User Name(%s)", vars["name"])
	profiler.Add(profiler_API_PROCESS_REQUEST, time.Duration(time.Since(tStart).Microseconds()))
}

// API Tag

func apiTag(w http.ResponseWriter, r *http.Request) {
	tStart := time.Now()
	fmt.Fprintf(w, "API Tag")
	profiler.Add(profiler_API_PROCESS_REQUEST, time.Duration(time.Since(tStart).Microseconds()))
}

func apiTagName(w http.ResponseWriter, r *http.Request) {
	tStart := time.Now()
	vars := mux.Vars(r)
	fmt.Fprintf(w, "API Tag Name(%s)", vars["name"])
	profiler.Add(profiler_API_PROCESS_REQUEST, time.Duration(time.Since(tStart).Microseconds()))
}

// API Tag

func apiDebugProfiler(w http.ResponseWriter, r *http.Request) {
	tStart := time.Now()

	var entries []string
	profiler.GetAll(&entries)

	for i := range entries {
		var entry string = entries[i]
		fmt.Fprintf(w, "API Debug Profiler(%s)\n\n%s\n\n", entry, string(profiler.Get(entry).ToString()))
	}

	profiler.Add(profiler_API_PROCESS_REQUEST, time.Duration(time.Since(tStart).Microseconds()))
}

func apiDebugProfilerName(w http.ResponseWriter, r *http.Request) {
	tStart := time.Now()
	vars := mux.Vars(r)
	fmt.Fprintf(w, "API Debug Profiler(%s)\n%s", vars["name"], string(profiler.Get(vars["name"]).ToString()))
	profiler.Add(profiler_API_PROCESS_REQUEST, time.Duration(time.Since(tStart).Microseconds()))
}
